from PIL.Image import Image, fromarray
import PIL.ImageDraw as ImageDraw
import PIL.ImageFont as ImageFont
from PIL import ImageColor
import numpy as np
import cv2
import os

STANDARD_COLORS = [
    'AliceBlue', 'Chartreuse', 'Aqua', 'Aquamarine', 'Azure', 'Beige', 'Bisque',
    'BlanchedAlmond', 'BlueViolet', 'BurlyWood', 'CadetBlue', 'AntiqueWhite',
    'Chocolate', 'Coral', 'CornflowerBlue', 'Cornsilk', 'Crimson', 'Cyan',
    'DarkCyan', 'DarkGoldenRod', 'DarkGrey', 'DarkKhaki', 'DarkOrange',
    'DarkOrchid', 'DarkSalmon', 'DarkSeaGreen', 'DarkTurquoise', 'DarkViolet',
    'DeepPink', 'DeepSkyBlue', 'DodgerBlue', 'FireBrick', 'FloralWhite',
    'ForestGreen', 'Fuchsia', 'Gainsboro', 'GhostWhite', 'Gold', 'GoldenRod',
    'Salmon', 'Tan', 'HoneyDew', 'HotPink', 'IndianRed', 'Ivory', 'Khaki',
    'Lavender', 'LavenderBlush', 'LawnGreen', 'LemonChiffon', 'LightBlue',
    'LightCoral', 'LightCyan', 'LightGoldenRodYellow', 'LightGray', 'LightGrey',
    'LightGreen', 'LightPink', 'LightSalmon', 'LightSeaGreen', 'LightSkyBlue',
    'LightSlateGray', 'LightSlateGrey', 'LightSteelBlue', 'LightYellow', 'Lime',
    'LimeGreen', 'Linen', 'Magenta', 'MediumAquaMarine', 'MediumOrchid',
    'MediumPurple', 'MediumSeaGreen', 'MediumSlateBlue', 'MediumSpringGreen',
    'MediumTurquoise', 'MediumVioletRed', 'MintCream', 'MistyRose', 'Moccasin',
    'NavajoWhite', 'OldLace', 'Olive', 'OliveDrab', 'Orange', 'OrangeRed',
    'Orchid', 'PaleGoldenRod', 'PaleGreen', 'PaleTurquoise', 'PaleVioletRed',
    'PapayaWhip', 'PeachPuff', 'Peru', 'Pink', 'Plum', 'PowderBlue', 'Purple',
    'Red', 'RosyBrown', 'RoyalBlue', 'SaddleBrown', 'Green', 'SandyBrown',
    'SeaGreen', 'SeaShell', 'Sienna', 'Silver', 'SkyBlue', 'SlateBlue',
    'SlateGray', 'SlateGrey', 'Snow', 'SpringGreen', 'SteelBlue', 'GreenYellow',
    'Teal', 'Thistle', 'Tomato', 'Turquoise', 'Violet', 'Wheat', 'White',
    'WhiteSmoke', 'Yellow', 'YellowGreen'
]
# 寻找列表中最大元素的下标


def find_max_index(lst):
    max_value = max(lst)
    max_index = lst.index(max_value)
    return max_index


def draw_text(draw,
              box: list,
              cls: int,
              score: float,
              mask: list,
              category_index: dict,
              color: str,
              font: str = 'arial.ttf',
              font_size: int = 30,
              ):
    """
    将目标边界框和类别信息绘制到图片上
    """
    font_path = os.path.join(cv2.__path__[0], 'qt', 'fonts', 'DejaVuSans.ttf')
    # try:
    font = ImageFont.truetype(font_path, font_size)
    # except IOError:
    #     font = ImageFont.load_default()
    # 计算脱矿区域面积
    area = 0
    for row in range(mask.shape[0]):
        for column in range(mask.shape[1]):
            if mask[row][column] != 0:
                area = area + 1
    left, top, right, bottom = box
    # If the total height of the display strings added to the top of the bounding
    # box exceeds the top of the image, stack the strings below the bounding box
    # instead of above.
    # 展示类别信息和概率分数
    # display_str = f"{category_index[str(cls)]}: {int(100 * score)}%"
    # 展示脱矿区域面积
    display_str = f"{area}"
    display_str_heights = [font.getsize(ds)[1] for ds in display_str]
    # Each display_str has a top and bottom margin of 0.05x.
    display_str_height = (1 + 2 * 0.05) * max(display_str_heights)

    if top > display_str_height:
        text_top = top - display_str_height
        text_bottom = top
    else:
        text_top = bottom
        text_bottom = bottom + display_str_height

    for ds in display_str:
        text_width, text_height = font.getsize(ds)
        margin = np.ceil(0.05 * text_width)
        draw.rectangle([(left, text_top),
                        (left + text_width + 2 * margin, text_bottom)], fill=color)
        draw.text((left + margin, text_top),
                  ds,
                  fill='black',
                  font=font)
        left += text_width

# 匹配牙齿和脱矿区域


def draw_teethAndCa(draw,
                    box: list,
                    annotations: list,
                    color: str,
                    mask1: list,
                    font: str = 'arial.ttf',
                    font_size: int = 50,
                    ):
    font = 'arial.ttf'
    font_path = os.path.join(cv2.__path__[0], 'qt', 'fonts', 'DejaVuSans.ttf')
    # try:
    font = ImageFont.truetype(font_path, font_size)
    boxesTeeth = []
    masksTeeth = []
    ious = []
    index = 0
    # print('即将计算iou')
    mask1 = mask1
    for i in range(len(annotations)):
        boxesTeeth.append(annotations[i]['bbox'])
        masksTeeth.append(annotations[i]['segmentation'])
        mask2 = annotations[i]['segmentation']
        iou = np.sum(np.logical_and(mask1, mask2)) / \
            np.sum(np.logical_or(mask1, mask2))
        ious.append(iou)
    index = find_max_index(ious)
    left, top, right, bottom = boxesTeeth[index]
    # 对应的牙齿面积
    area = np.sum(masksTeeth[index])
    display_str = f"area: {area}"
    display_str_heights = [font.getsize(ds)[1] for ds in display_str]
    # Each display_str has a top and bottom margin of 0.05x.
    display_str_height = (1 + 2 * 0.05) * max(display_str_heights)

    if top > display_str_height:
        text_top = top - display_str_height
        text_bottom = top
    else:
        text_top = bottom
        text_bottom = bottom + display_str_height

    for ds in display_str:
        text_width, text_height = font.getsize(ds)
        margin = np.ceil(0.05 * text_width)
        draw.rectangle([(left, text_top),
                        (left + text_width + 2 * margin, text_bottom)], fill=color)
        draw.text((left + margin, text_top),
                  ds,
                  fill='black',
                  font=font)
        left += text_width
    # draw.line([(left, top), (left, bottom), (right, bottom),
    #                    (right, top), (left, top)], width=8, fill=color)
    return boxesTeeth[index], index


# 实现脱矿区域实例分割效果
def draw_masks(image, masks, colors, thresh: float = 0.7, alpha: float = 0.8):
    np_image = np.array(image)
    masks = np.where(masks > thresh, True, False)

    # colors = np.array(colors)
    img_to_draw = np.copy(np_image)
    # TODO: There might be a way to vectorize this
    for mask, color in zip(masks, colors):
        img_to_draw[mask] = color

    out = np_image * (1 - alpha) + img_to_draw * alpha
    return fromarray(out.astype(np.uint8))

# 实现牙齿实例分割效果


def draw_masksTeeth(image: Image,
                    annotations: list,
                    index: list):
    np_image = np.array(image)
    masks = []
    for i in index:
        masks.append(annotations[i]['segmentation'])
    color_mask = []
    print(f'分割出牙齿的mask个数{len(masks)}')
    for i in range(len(masks)):
        mask = np.random.random((1, 3)).tolist()[0]
        for j in range(len(mask)):
            mask[j] = int(mask[j] * 255)
        color_mask.append(mask)
    img_to_draw = np.copy(np_image)
    for mask, color in zip(masks, color_mask):
        img_to_draw[mask] = color
    out = np_image * (1 - 0.5) + img_to_draw * 0.5
    return fromarray(out.astype(np.uint8)), masks


def draw_objs(image: Image,
              boxes: np.ndarray = None,
              classes: np.ndarray = None,
              scores: np.ndarray = None,
              masks: np.ndarray = None,
              masks2: list = None,
              category_index: dict = None,
              box_thresh: float = 0.1,
              mask_thresh: float = 0.5,
              line_thickness: int = 8,
              font: str = 'arial.ttf',
              font_size: int = 500,
              draw_boxes_on_image: bool = True,
              draw_masks_on_image: bool = True):
    """
    将目标边界框信息，类别信息，mask信息绘制在图片上
    Args:
        image: 需要绘制的图片
        boxes: 目标边界框信息
        classes: 目标类别信息
        scores: 目标概率信息
        masks: 目标mask信息
        masks2: 目标mask2信息
        category_index: 类别与名称字典
        box_thresh: 过滤的概率阈值
        mask_thresh:
        line_thickness: 边界框宽度
        font: 字体类型
        font_size: 字体大小
        draw_boxes_on_image:
        draw_masks_on_image:

    Returns:

    """

    # 过滤掉低概率的目标
    idxs = np.greater(scores, box_thresh)
    boxes = boxes[idxs]
    classes = classes[idxs]
    scores = scores[idxs]
    if masks is not None:
        masks = masks[idxs]
    if len(boxes) == 0:
        return image
    colors = [ImageColor.getrgb(
        STANDARD_COLORS[cls % len(STANDARD_COLORS)]) for cls in classes]
    # print(classes+'1111')
    if draw_boxes_on_image:
        # Draw all boxes onto image.
        teethIndex = []
        draw = ImageDraw.Draw(image)
        for box, cls, score, color, mask in zip(boxes, classes, scores, colors, masks):
            left, top, right, bottom = box
            # 绘制目标边界框
            # draw.line([(left, top), (left, bottom), (right, bottom),
            #            (right, top), (left, top)], width=line_thickness, fill=color)
            # 绘制类别和概率信息
            draw_text(draw, box.tolist(), int(cls), float(score),
                      mask, category_index, color, font, font_size)
            # 进行脱矿区域和牙齿匹配，返回匹配牙齿的边界框坐标和在mask2中的index
            boxTeeth, toothIndex = draw_teethAndCa(
                draw, box.tolist(), masks2, color, mask, font, font_size)
            # print('计算iou完成')
            left, top = boxTeeth[0], boxTeeth[1]
            right = left + boxTeeth[2]
            bottom = top + boxTeeth[3]
            # draw.line([(left, top), (left, bottom), (right, bottom),
            #            (right, top), (left, top)], width=line_thickness, fill=color)
            # if toothIndex not in teethIndex:
            teethIndex.append(toothIndex)
    if draw_masks_on_image and (masks is not None):
        # Draw all mask onto image.
        image = draw_masks(image, masks, colors, mask_thresh)
    # 画出实例分割后的牙齿
    print(f'所匹配的牙齿编号为{teethIndex}，下面开始绘制')
    image, teethMasks = draw_masksTeeth(image, masks2, teethIndex)

    return image, teethMasks
